import { objectGet, objectLoop, objectSet } from '@dzeio/object-util'
import SchemaArray from 'items/array'
import SchemaBoolean from 'items/boolean'
import SchemaNullable from 'items/nullable'
import SchemaObject from 'items/object'
import type SchemaItem from 'SchemaItem'

export function isNull(value: unknown): value is undefined | null {
	return typeof value === 'undefined' || value === null
}

export function parseQuery<T extends SchemaItem>(model: T, query: URLSearchParams, opts?: Parameters<T['parse']>[1]): ReturnType<T['parse']> {
	const record: Record<string, unknown> = {}
	for (const [key, value] of query) {
		record[key] = value
	}

	return model.parse(record, opts) as ReturnType<T['parse']>
}

export function parseFormData<T extends SchemaObject>(model: T, data: FormData, opts?: Parameters<T['parse']>[1]): ReturnType<T['parse']> {
	const record: Record<string, unknown> = {}
	// console.log('VALIDATE FORM DATA data', data)
	for (const [key, value] of data) {
		// console.log('parse', key, value)
		const isArray = model.model[key].isOfType([]) ?? false
		// record[key] = isArray ? data.getAll(key) : value
		objectSet(record, key.split('.').map((it) => /^\d+$/g.test(it) ? parseInt(it, 10) : it), isArray ? data.getAll(key) : value)
	}

	// quick hack to handle FormData not returning Checkboxes
	const handleBoolean = (value: SchemaItem, keys: Array<string | number>) => {
		if (value instanceof SchemaNullable) {
			handleBoolean(value.unwrap(), keys)
		}

		if (value instanceof SchemaArray) {
			const elements: Array<unknown> | undefined = objectGet(record, keys)
			for (let it = 0; it < (elements?.length ?? 0); it++) {
				handleBoolean(value.unwrap(), [...keys, it])
			}
		}

		if (value instanceof SchemaObject) {
			handleSchemaForBoolean(value.model as Record<string, SchemaItem>, keys)
		}

		if (value instanceof SchemaBoolean) {
			objectSet(record, keys, !!data.get(keys.join('.')))
		}
	}
	const handleSchemaForBoolean = (modl: Record<string, SchemaItem>, keys: Array<string | number> = []) => {
		objectLoop(modl, (value, key) => {
			handleBoolean(value as unknown as SchemaItem, [...keys, key])
		})
	}
	handleSchemaForBoolean(model.model)
	// console.log(JSON.stringify(record, undefined, 2))
	return model.parse(record, opts) as ReturnType<T['parse']>
}

export function parseForm<T extends SchemaObject>(model: T, form: HTMLFormElement, opts?: Parameters<T['parse']>[1]): ReturnType<T['parse']> {
	return parseFormData(model, new FormData(form), opts)
}
